@@ -8,7 +8,7 @@
 
 use std::os::fd::{AsFd, AsRawFd, FromRawFd, IntoRawFd, RawFd};
 
-use libseccomp::ScmpNotifResp;
+use libseccomp::{ScmpArch, ScmpNotifResp};
 use nix::{
     errno::Errno,
     sys::fanotify::{Fanotify, MarkFlags, MaskFlags},
@@ -16,13 +16,14 @@ use nix::{
 
 use crate::{
     config::PROC_FILE,
+    confine::scmp_arch_bits,
     fs::{fd_mode, is_sidechannel_device, FsFlags},
     hook::{SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
     path::XPathBuf,
-    scmp_arch_bits,
 };
 
+#[allow(clippy::arithmetic_side_effects)]
 pub(crate) fn sys_fanotify_mark(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
@@ -39,7 +40,22 @@ pub(crate) fn sys_fanotify_mark(request: UNotifyEventRequest) -> ScmpNotifResp {
     };
 
     // SAFETY: Reject undefined/invalid masks.
-    let mask = match MaskFlags::from_bits(req.data.args[2]) {
+    // Note, mask is a 64-bit value but kernel uses the lower 32-bits only.
+    let is32 = scmp_arch_bits(req.data.arch) == 32;
+    #[allow(clippy::cast_possible_truncation)]
+    #[allow(clippy::cast_possible_wrap)]
+    let (mask, narg) = if is32 {
+        let mask = match req.data.arch {
+            ScmpArch::Ppc => req.data.args[3],
+            _ => req.data.args[2],
+        };
+
+        (mask, 3)
+    } else {
+        (req.data.args[2], 2)
+    };
+
+    let mask = match MaskFlags::from_bits(mask) {
         Some(mask) => mask,
         None => return request.fail_syscall(Errno::EINVAL),
     };
@@ -53,10 +69,9 @@ pub(crate) fn sys_fanotify_mark(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags |= FsFlags::NO_FOLLOW_LAST;
     }
 
-    let is32 = scmp_arch_bits(req.data.arch) == 32;
-    let pidx = if is32 { 5 } else { 4 };
+    let pidx = narg + 2;
     let argv = &[SysArg {
-        dirfd: Some(if is32 { 4 } else { 3 }),
+        dirfd: Some(narg + 1),
         path: if req.data.args[pidx] != 0 {
             Some(pidx)
         } else {
