@@ -6,23 +6,26 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::os::fd::{AsRawFd, OwnedFd};
+use std::os::fd::OwnedFd;
 
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    sys::socket::{connect, SockaddrStorage},
+    sys::socket::{AddressFamily, SockaddrLike, SockaddrStorage},
 };
 
 use crate::{
+    cookie::safe_connect,
     fs::{get_nonblock, has_recv_timeout},
     hook::UNotifyEventRequest,
+    kernel::net::handle_safe_bind,
 };
 
 pub(crate) fn handle_connect(
     fd: OwnedFd,
     addr: &SockaddrStorage,
     request: &UNotifyEventRequest,
+    allow_safe_bind: bool,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Record blocking call so it can get invalidated.
     let req = request.scmpreq;
@@ -37,12 +40,25 @@ pub(crate) fn handle_connect(
         false
     };
 
-    let result = connect(fd.as_raw_fd(), addr).map(|_| request.return_syscall(0));
-    drop(fd); // Close our copy of the socket.
+    let result = safe_connect(&fd, addr).map(|_| request.return_syscall(0));
 
     // Remove invalidation record unless interrupted.
-    if is_blocking && !matches!(result, Err(Errno::EINTR)) {
-        request.cache.del_sys_block(req.id);
+    if is_blocking {
+        request
+            .cache
+            .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
+    }
+
+    // Handle allow_safe_bind.
+    // Ignore errors as connect has already succeeded.
+    if allow_safe_bind
+        && result.is_ok()
+        && matches!(
+            addr.family(),
+            Some(AddressFamily::Inet | AddressFamily::Inet6)
+        )
+    {
+        let _ = handle_safe_bind(request, &fd);
     }
 
     result
