@@ -16,13 +16,14 @@ use nix::{
     sys::socket::{sendmsg, ControlMessage, MsgFlags, SockaddrStorage, UnixCredentials},
     unistd::Pid,
 };
-use secure_string::SecureBytes;
+use zeroize::Zeroizing;
 
 use crate::{
     compat::{
         addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, PF_ALG, PF_INET, PF_INET6,
         PF_NETLINK, PF_UNIX,
     },
+    confine::scmp_arch_bits,
     fs::{file_type, get_nonblock, has_recv_timeout, is_same_vm, FileType},
     hook::UNotifyEventRequest,
     kernel::{
@@ -31,7 +32,6 @@ use crate::{
     },
     path::XPath,
     sandbox::Capability,
-    scmp_arch_bits,
 };
 
 #[allow(clippy::cognitive_complexity)]
@@ -41,10 +41,20 @@ pub(crate) fn handle_sendmsg(
     request: &UNotifyEventRequest,
     args: &[u64; 6],
     allow_unsupp_socket: bool,
+    restrict_oob: bool,
+    restrict_mkbdev: bool,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
     let flags = to_msgflags(args[2])?;
 
+    // SAFETY: Reject MSG_OOB as necessary.
+    if restrict_oob && flags.contains(MsgFlags::MSG_OOB) {
+        // Signal no support to let the sandbox process
+        // handle the error gracefully. This is consistent
+        // with the Linux kernel.
+        return Err(Errno::EOPNOTSUPP);
+    }
+
     let req = request.scmpreq;
     let is32 = scmp_arch_bits(req.data.arch) == 32;
     let size = if is32 {
@@ -87,7 +97,6 @@ pub(crate) fn handle_sendmsg(
             &sandbox,
             get_addr(request, addr_remote as u64, addr_len)?,
             Capability::CAP_NET_CONNECT,
-            *sandbox.flags,
         )?;
         match addr_family(&addr) {
             PF_UNIX | PF_INET | PF_INET6 => {
@@ -178,7 +187,7 @@ pub(crate) fn handle_sendmsg(
 
         for fds in control_fds {
             // SAFETY: Deny sending file descriptors referring to
-            // 1. Block devices
+            // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
             // 2. Directories
             // 3. Symbolic links
             //
@@ -187,12 +196,11 @@ pub(crate) fn handle_sendmsg(
             // depend on this. See test-pw-filter test of pipewire
             // for more information about this.
             for fd in fds {
-                if matches!(
-                    file_type(fd, None, false)?,
-                    FileType::Blk | FileType::Dir | FileType::Lnk
-                ) {
-                    // TODO: Log this deny!
-                    return Err(Errno::EACCES);
+                // TODO: Log this deny!
+                match file_type(fd, None, false)? {
+                    FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
+                    FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
+                    _ => {}
                 }
             }
 
@@ -209,7 +217,7 @@ pub(crate) fn handle_sendmsg(
         }
 
         for iv_data in control_ivs {
-            control_messages.push(ControlMessage::AlgSetIv(iv_data.unsecure()));
+            control_messages.push(ControlMessage::AlgSetIv(iv_data.as_slice()));
         }
 
         for op in control_ops {
@@ -367,10 +375,20 @@ pub(crate) fn handle_sendmmsg(
     request: &UNotifyEventRequest,
     args: &[u64; 6],
     allow_unsupp_socket: bool,
+    restrict_oob: bool,
+    restrict_mkbdev: bool,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
     let msgflags = to_msgflags(args[3])?;
 
+    // SAFETY: Reject MSG_OOB as necessary.
+    if restrict_oob && msgflags.contains(MsgFlags::MSG_OOB) {
+        // Signal no support to let the sandbox process
+        // handle the error gracefully. This is consistent
+        // with the Linux kernel.
+        return Err(Errno::EOPNOTSUPP);
+    }
+
     // NULL check was performed already.
     let addr = args[1];
     let vlen = usize::try_from(args[2]).or(Err(Errno::EINVAL))?;
@@ -434,7 +452,6 @@ pub(crate) fn handle_sendmmsg(
                 &sandbox,
                 get_addr(request, addr_remote as u64, addr_len)?,
                 Capability::CAP_NET_CONNECT,
-                *sandbox.flags,
             )?;
             match addr_family(&addr) {
                 PF_UNIX | PF_INET | PF_INET6 => {
@@ -528,7 +545,7 @@ pub(crate) fn handle_sendmmsg(
 
                 for fds in control_fds {
                     // SAFETY: Deny sending file descriptors referring to
-                    // 1. Block devices
+                    // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
                     // 2. Directories
                     // 3. Symbolic links
                     //
@@ -537,12 +554,11 @@ pub(crate) fn handle_sendmmsg(
                     // depend on this. See test-pw-filter test of pipewire
                     // for more information about this.
                     for fd in fds {
-                        if matches!(
-                            file_type(fd, None, false)?,
-                            FileType::Blk | FileType::Dir | FileType::Lnk
-                        ) {
-                            // TODO: Log this deny!
-                            return Err(Errno::EACCES);
+                        // TODO: Log this deny!
+                        match file_type(fd, None, false)? {
+                            FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
+                            FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
+                            _ => {}
                         }
                     }
 
@@ -560,7 +576,7 @@ pub(crate) fn handle_sendmmsg(
                 }
 
                 for iv_data in control_ivs {
-                    control_messages.push(ControlMessage::AlgSetIv(iv_data.unsecure()));
+                    control_messages.push(ControlMessage::AlgSetIv(iv_data.as_slice()));
                 }
 
                 for op in control_ops {
@@ -733,10 +749,10 @@ fn parse_control_messages(
     (
         Vec<Vec<OwnedFd>>,
         Vec<UnixCredentials>,
-        Vec<SecureBytes>, // ivs
-        Vec<libc::c_int>, // ops
-        Vec<u32>,         // aead_assoclens
-        Vec<u16>,         // udp_gso_segments
+        Vec<Zeroizing<Vec<u8>>>, // ivs
+        Vec<libc::c_int>,        // ops
+        Vec<u32>,                // aead_assoclens
+        Vec<u16>,                // udp_gso_segments
         Vec<libc::in_pktinfo>,
         Vec<libc::in6_pktinfo>,
         Vec<u32>, // rxq_ovfls
@@ -873,7 +889,7 @@ fn parse_control_messages(
                 if iv_size <= data_len {
                     // Extract the IV while respecting the indicated size,
                     // if the size is valid.
-                    let iv_data = SecureBytes::from(&data[4..iv_size]);
+                    let iv_data = Zeroizing::new(data[4..iv_size].to_vec());
                     control_ivs.push(iv_data); // Store the IV data.
                 } else {
                     return Err(Errno::EINVAL); // Data length mismatch.
