@@ -101,7 +101,7 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             std::ptr::null()
         };
 
-        if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+        if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             // SAFETY: Deny user.syd* extended attributes. name is either
             // NULL or a valid nul-terminated C-String.
             // SAFETY: Deny with ENODATA for stealth.
@@ -254,7 +254,7 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             std::ptr::null()
         };
 
-        if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+        if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             // SAFETY: Deny user.syd* extended attributes. name is either
             // NULL or a valid nul-terminated C-String.
             // SAFETY: Deny with ENODATA for stealth.
@@ -395,7 +395,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             let n = if let Some(buf) = buf {
                 // SAFETY: Filter out attributes that start with "user.syd".
                 // SAFETY: Deny only if the Sandbox is locked for the process.
-                let buf = if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+                let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
                     Cow::Owned(filterxattr(&buf[..n], n)?)
                 } else {
                     Cow::Borrowed(&buf[..n])
@@ -563,7 +563,7 @@ pub(crate) fn sys_lremovexattr(request: UNotifyEventRequest) -> ScmpNotifResp {
                 std::ptr::null()
             };
 
-            if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+            if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
                 // SAFETY: Deny user.syd* extended attributes.
                 // name is either NULL or a valid nul-terminated C-String.
                 // SAFETY: Deny with ENODATA for stealth.
@@ -665,7 +665,7 @@ fn syscall_getxattr_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
         // NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with ENODATA for stealth.
@@ -785,7 +785,7 @@ fn syscall_getxattrat_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
         // NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with ENODATA for stealth.
@@ -872,7 +872,7 @@ fn syscall_setxattr_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
         // NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with EACCES to denote access violation.
@@ -960,7 +960,7 @@ fn syscall_setxattrat_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes. name is either
         // NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with EACCES to denote access violation.
@@ -1064,7 +1064,7 @@ fn syscall_listxattr_handler(
     if let Some(buf) = buf {
         // SAFETY: Filter out attributes that start with "user.syd".
         // SAFETY: Deny only if the Sandbox is locked for the process.
-        let buf = if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+        let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             Cow::Owned(filterxattr(&buf[..n], n)?)
         } else {
             Cow::Borrowed(&buf[..n])
@@ -1135,7 +1135,7 @@ fn syscall_listxattrat_handler(
     if let Some(buf) = buf {
         // SAFETY: Filter out attributes that start with "user.syd".
         // SAFETY: Deny only if the Sandbox is locked for the process.
-        let buf = if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+        let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
             Cow::Owned(filterxattr(&buf[..n], n)?)
         } else {
             Cow::Borrowed(&buf[..n])
@@ -1178,7 +1178,7 @@ fn syscall_removexattr_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes.
         // name is either NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with ENODATA for stealth.
@@ -1224,7 +1224,7 @@ fn syscall_removexattrat_handler(
         std::ptr::null()
     };
 
-    if !sandbox.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
+    if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
         // SAFETY: Deny user.syd* extended attributes.
         // name is either NULL or a valid nul-terminated C-String.
         // SAFETY: Deny with ENODATA for stealth.
